diff -uN bogohist.c bogohist.c
--- bogohist.c	1969-12-31 19:00:00.000000000 -0500
+++ bogohist.c	2004-01-03 09:14:25.000000000 -0500
@@ -0,0 +1,175 @@
+/* $Id: bogohist.c,v 1.1 2004/01/03 14:09:03 relson Exp $ */
+
+/*****************************************************************************
+
+NAME:
+   bogoutil.c -- dumps and loads bogofilter text files from/to Berkeley DB format.
+
+AUTHOR:
+   Gyepi Sam <gyepi@praxis-sw.com>
+   
+******************************************************************************/
+
+#include "common.h"
+
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
+#include <errno.h>
+#include <sys/stat.h>
+
+#include "bogohist.h"
+#include "prob.h"
+#include "datastore.h"
+#include "word.h"
+#include "wordlists.h"
+
+uint msgs_good, msgs_bad;
+uint ham_only,  ham_hapax;
+uint spam_only, spam_hapax;
+uint token_count;
+
+#define	INTERVALS	20
+#define PCT(n)		100.0 * n / token_count
+
+typedef struct rhistogram_s rhistogram_t;
+struct rhistogram_s {
+    uint32_t count[INTERVALS];
+};
+
+/* Function Prototypes */
+
+/* Function Definitions */
+
+static int ds_histogram_hook(/*@unused@*/ word_t *key, dsv_t *data,
+			     void *userdata)
+/* returns 0 if ok, 1 if not ok */
+{
+    rhistogram_t *hist = userdata;
+
+    double fw = calc_prob(data->goodcount, data->spamcount);
+    uint idx = min(fw * INTERVALS, INTERVALS-1);
+
+    (void)key;
+
+    hist->count[idx] += 1;
+
+    if (data->spamcount == 0) {
+	ham_only += 1;
+	if (data->goodcount == 1)
+	    ham_hapax += 1;
+    }
+
+    if (data->goodcount == 0) {
+	spam_only += 1;
+	if (data->spamcount == 1)
+	    spam_hapax += 1;
+    }
+
+    return 0;
+}
+
+static int print_histogram(rhistogram_t *hist)
+{
+    uint i, r;
+    uint maxcnt = 0;
+
+    (void)printf("%5s%8s  %3s  %s\n", "score", "count", "pct", "histogram");
+
+    for (i=0; i<INTERVALS; i+=1)
+    {
+	uint32_t cnt = hist->count[i];
+	if (cnt > maxcnt) 
+	    maxcnt = cnt;
+	token_count += cnt;
+    }
+
+    /* Print histogram */
+    for (i=0; i<INTERVALS; i+=1)
+    {
+	uint32_t cnt = hist->count[i];
+	double beg = 1.0 * i / INTERVALS;
+	double pct = PCT(cnt);
+
+	/* print interval, count, probability, percent, and spamicity */
+	(void)printf("%3.2f %8u %5.2f ", beg, cnt, pct);
+
+	/* scale histogram to 48 characters */
+	if (maxcnt>48) cnt = (cnt * 48 + maxcnt - 1) / maxcnt;
+
+	/* display histogram */
+	for (r=0; r<cnt; r+=1)
+	    (void)fputc( '#', stdout);
+	(void)fputc( '\n', stdout);
+    }
+
+    (void)printf("tot  %8u\n", token_count);
+
+    return 0;
+}
+
+int histogram(const char *path)
+{
+    int rc;
+    uint count;
+    void *dsh;
+    dsv_t val;
+
+    rhistogram_t hist;
+
+    char filepath1[PATH_LEN];
+    char filepath2[PATH_LEN];
+    char *filepaths[IX_SIZE];
+    filepaths[0] = filepath1;
+    filepaths[1] = filepath2;
+
+    memset(&hist, 0, sizeof(hist));
+
+#ifdef	ENABLE_DEPRECATED_CODE
+    set_wordlist_mode(path);
+#endif
+
+    count = build_wordlist_paths(filepaths, path);
+
+    dsh = ds_open(CURDIR_S, count, (const char **)filepaths, DB_READ);
+    if (dsh == NULL)
+	return EX_ERROR;
+
+    ds_get_msgcounts(dsh, &val);
+    msgs_bad  = val.spamcount;
+    msgs_good = val.goodcount;
+    
+    ds_close(dsh, false);
+
+    rc = ds_oper(filepaths[0], DB_READ, ds_histogram_hook, &hist);
+
+    print_histogram(&hist);
+
+    printf("hapaxes:  ham %7d (%5.2f%%), spam %7d (%5.2f%%)\n", ham_hapax, PCT(ham_hapax), spam_hapax, PCT(spam_hapax));
+    printf("   pure:  ham %7d (%5.2f%%), spam %7d (%5.2f%%)\n", ham_only,  PCT(ham_only),  spam_only,  PCT(spam_only));
+
+    return rc;
+}
+
+/* for a standalone program:
+**
+**	cc -o bogohist.prog.o -DMAIN -c bogohist.c
+**	cc -o bogohist bogohist.prog.o libbogofilter.a strlcpy.o strlcat.o -ldb  -lm
+*/
+
+#ifdef	MAIN
+const char *progname = "bogohist";
+
+int main(int argc, char *argv[])
+{
+    if (argc < 2) {
+	fprintf(stderr, "usage: %s BOGOFILTER_DIR\n", progname);
+	exit(1);
+    }
+    else {
+	const char *path = argv[1];
+	int rc = histogram(path);
+	exit(rc);
+    }
+}
+#endif
diff -uN bogohist.h bogohist.h
--- bogohist.h	1969-12-31 19:00:00.000000000 -0500
+++ bogohist.h	2004-01-03 09:09:30.000000000 -0500
@@ -0,0 +1,19 @@
+/* $Id: bogohist.h,v 1.1 2004/01/03 14:09:03 relson Exp $ */
+
+/*****************************************************************************
+
+NAME:
+   histogram.h -- constants and declarations for histogram.c
+
+AUTHOR:
+   David Relson <relson@osagesoftware.com>
+
+******************************************************************************/
+
+#ifndef	HISTOGRAM_H
+#define	HISTOGRAM_H
+
+extern int histogram(const char *path);
+
+#endif	/* HISTOGRAM_H */
+
diff -uN bogoutil.c bogoutil.c
--- bogoutil.c	2004-01-01 08:34:49.000000000 -0500
+++ bogoutil.c	2004-01-03 09:09:03.000000000 -0500
@@ -1,4 +1,4 @@
-/* $Id: bogoutil.c,v 1.84 2004/01/01 13:34:49 relson Exp $ */
+/* $Id: bogoutil.c,v 1.85 2004/01/03 14:09:03 relson Exp $ */
 
 /*****************************************************************************
 
@@ -19,6 +19,7 @@
 #include <sys/stat.h>
 
 #include "bogofilter.h"
+#include "bogohist.h"
 #include "buff.h"
 #include "datastore.h"
 #include "error.h"
@@ -405,13 +406,14 @@
     usage();
     fprintf(stderr,
 	    "\n"
+	    "\t-h\tPrint this message.\n"
 	    "\t-d file\tDump data from file to stdout.\n"
 	    "\t-l file\tLoad data from stdin into file.\n"
 	    "\t-w dir\tDisplay counts for words from stdin.\n"
 	    "\t-p dir\tDisplay word counts and probabilities.\n"
 	    "\t-m\tEnable maintenance works (expiring tokens).\n"
-	    "\t-h\tPrint this message.\n"
 	    "\t-v\tOutput debug messages.\n"
+	    "\t-H dir\tDisplay histogram and statistics for the wordlist.\n"
 	    "\t-r dir\tCompute Robinson's X for specified directory.\n"
 	    "\t-R dir\tCompute Robinson's X and save it in the wordlist.\n");
     fprintf(stderr,
@@ -439,13 +441,13 @@
 char *ds_file = NULL;
 bool  prob = false;
 
-typedef enum { M_NONE, M_DUMP, M_LOAD, M_WORD, M_MAINTAIN, M_ROBX } cmd_t;
+typedef enum { M_NONE, M_DUMP, M_LOAD, M_WORD, M_MAINTAIN, M_ROBX, M_HIST } cmd_t;
 cmd_t flag = M_NONE;
 
 #ifndef	ENABLE_DEPRECATED_CODE
-#define	OPTIONS	":a:c:d:DhI:k:l:m:np:r:R:s:vVw:x:y:"
+#define	OPTIONS	":a:c:d:DhH:I:k:l:m:np:r:R:s:vVw:x:y:"
 #else
-#define	OPTIONS	":a:c:d:DhI:k:l:m:np:r:R:s:vVw:Wx:y:"
+#define	OPTIONS	":a:c:d:DhH:I:k:l:m:np:r:R:s:vVw:Wx:y:"
 #endif
 
 static int process_args(int argc, char **argv)
@@ -514,6 +516,12 @@
 	    help();
 	    exit(EX_OK);
 
+	case 'H':
+	    flag = M_HIST;
+	    count += 1;
+	    ds_file = (char *) optarg;
+	    break;
+
 	case 'V':
 	    print_version();
 	    exit(EX_OK);
@@ -626,6 +634,8 @@
 	    argc -= optind;
 	    argv += optind;
 	    return display_words(ds_file, argc, argv, prob);
+	case M_HIST:
+	    return histogram(ds_file);
 	case M_ROBX:
 	    return get_robx(ds_file);
 	case M_NONE:
Common subdirectories: CVS and CVS
diff -uN Makefile.am Makefile.am
--- Makefile.am	2004-01-01 08:34:49.000000000 -0500
+++ Makefile.am	2004-01-03 09:09:02.000000000 -0500
@@ -1,5 +1,5 @@
 # vim:tw=0
-# $Id: Makefile.am,v 1.71 2004/01/01 13:34:49 relson Exp $
+# $Id: Makefile.am,v 1.72 2004/01/03 14:09:02 relson Exp $
 
 SUBDIRS = . tests
 SYSCONFDIR = @sysconfdir@
@@ -151,7 +151,8 @@
 bogolexer_static_SOURCES = bogolexer.c
 bogolexer_static_LDFLAGS = $(STATICLDFLAGS)
 
-bogoutil_static_SOURCES = bogoutil.c
+bogoutil_SOURCES = bogoutil.c bogohist.c
+bogoutil_static_SOURCES = bogoutil.c bogohist.c
 bogoutil_static_LDFLAGS = $(STATICLDFLAGS)
 bogoutil_static_LDADD = $(LDADD) $(STATIC_DB)
 
